@@ -16,9 +16,7 @@
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #define _GNU_SOURCE
@@ -125,6 +123,8 @@ typedef struct _MetaCompositorXRender
   Atom atom_net_wm_window_type_dropdown_menu;
   Atom atom_net_wm_window_type_tooltip;
 
+  Atom atom_metacity_window_have_shadow;
+
 #ifdef USE_IDLE_REPAINT
   guint repaint_id;
 #endif
@@ -201,6 +201,7 @@ typedef struct _MetaCompWindow
   Picture picture;
   Picture alpha_pict;
 
+  gboolean have_shadow;
   gboolean needs_shadow;
   MetaShadowType shadow_type;
   Picture shadow_pict;
@@ -894,6 +895,9 @@ window_has_shadow (MetaCompWindow *cw)
   if (info == NULL || info->have_shadows == FALSE)
     return FALSE;
 
+  if (cw->have_shadow == FALSE)
+    return FALSE;
+
   /* Always put a shadow around windows with a frame - This should override
      the restriction about not putting a shadow around shaped windows
      as the frame might be the reason the window is shaped */
@@ -1131,8 +1135,7 @@ paint_windows (MetaScreen   *screen,
   MetaCompScreen *info = meta_screen_get_compositor_data (screen);
   Display *xdisplay = meta_display_get_xdisplay (display);
   GList *index, *last;
-  int screen_width, screen_height, screen_number;
-  Window xroot;
+  int screen_width, screen_height;
   MetaCompWindow *cw;
   XserverRegion paint_region, desktop_region;
 
@@ -1142,8 +1145,6 @@ paint_windows (MetaScreen   *screen,
     }
 
   meta_screen_get_size (screen, &screen_width, &screen_height);
-  screen_number = meta_screen_get_screen_number (screen);
-  xroot = meta_screen_get_xroot (screen);
 
   if (region == None) 
     {
@@ -1867,6 +1868,8 @@ add_win (MetaScreen *screen,
   
   cw->border_clip = None;
 
+  cw->have_shadow = TRUE;
+
   determine_mode (display, screen, cw);
   cw->needs_shadow = window_has_shadow (cw);
 
@@ -2232,6 +2235,51 @@ process_property_notify (MetaCompositorXRender *compositor,
         }
     }
 
+    /* Check for have_shadow changing */
+  if (event->atom == compositor->atom_metacity_window_have_shadow)
+    {
+      MetaCompWindow *cw = find_window_in_display (display, event->window);
+      gulong value;
+
+      if (!cw)
+        {
+          /* Applications can set this for their toplevel windows, so
+           * this must be propagated to the window managed by the compositor
+           */
+          cw = find_window_for_child_window_in_display (display, event->window);
+        }
+
+      if (!cw)
+        return;
+
+      if (meta_prop_get_cardinal (display, event->window,
+                                  compositor->atom_metacity_window_have_shadow,
+                                  &value) == FALSE)
+        value = 1;
+
+      cw->have_shadow = (gboolean) value;
+
+      determine_mode (display, cw->screen, cw);
+      cw->needs_shadow = window_has_shadow (cw);
+
+      if (cw->shadow)
+        {
+          XRenderFreePicture (xdisplay, cw->shadow);
+          cw->shadow = None;
+        }
+
+      if (cw->extents)
+        XFixesDestroyRegion (xdisplay, cw->extents);
+      cw->extents = win_extents (cw);
+
+      cw->damaged = TRUE;
+#ifdef USE_IDLE_REPAINT
+      add_repair (display);
+#endif
+
+      return;
+    }
+
   /* Check for the opacity changing */
   if (event->atom == compositor->atom_net_wm_window_opacity) 
     {
@@ -3026,7 +3074,8 @@ meta_compositor_xrender_new (MetaDisplay *display)
     "_NET_WM_WINDOW_TYPE_SPLASH",
     "_NET_WM_WINDOW_TYPE_TOOLBAR",
     "_NET_WM_WINDOW_TYPE_DROPDOWN_MENU",
-    "_NET_WM_WINDOW_TYPE_TOOLTIP"
+    "_NET_WM_WINDOW_TYPE_TOOLTIP",
+    "METACITY_WINDOW_HAVE_SHADOW",
   };
   Atom atoms[G_N_ELEMENTS(atom_names)];
   MetaCompositorXRender *xrc;
@@ -3059,6 +3108,9 @@ meta_compositor_xrender_new (MetaDisplay *display)
   xrc->atom_net_wm_window_type_toolbar = atoms[12];
   xrc->atom_net_wm_window_type_dropdown_menu = atoms[13];
   xrc->atom_net_wm_window_type_tooltip = atoms[14];
+  xrc->atom_metacity_window_have_shadow = atoms[15];
+  xrc->show_redraw = FALSE;
+  xrc->debug = FALSE;
 
 #ifdef USE_IDLE_REPAINT
   meta_verbose ("Using idle repaint\n");
