@@ -19,9 +19,7 @@
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -44,6 +42,7 @@
 #define KEY_NUM_WORKSPACES "num-workspaces"
 #define KEY_WORKSPACE_NAMES "workspace-names"
 #define KEY_COMPOSITOR "compositing-manager"
+#define KEY_PLACEMENT_MODE "placement-mode"
 
 /* Keys from "foreign" schemas */
 #define KEY_GNOME_ACCESSIBILITY "toolkit-accessibility"
@@ -89,11 +88,14 @@ static char *cursor_theme = NULL;
 static int   cursor_size = 24;
 static gboolean compositing_manager = FALSE;
 static gboolean resize_with_right_button = FALSE;
+static gboolean edge_tiling = FALSE;
 static gboolean force_fullscreen = TRUE;
 
 static GDesktopVisualBellType visual_bell_type = G_DESKTOP_VISUAL_BELL_FULLSCREEN_FLASH;
 static MetaButtonLayout button_layout;
 
+static MetaPlacementMode placement_mode = META_PLACEMENT_MODE_SMART;
+
 /* NULL-terminated array */
 static char **workspace_names = NULL;
 
@@ -244,6 +246,13 @@ static MetaEnumPreference preferences_enum[] =
       },
       &action_right_click_titlebar,
     },
+    {
+      { "placement-mode",
+        SCHEMA_METACITY,
+        META_PREF_PLACEMENT_MODE,
+      },
+      &placement_mode,
+    },
     { { NULL, 0, 0 }, NULL },
   };
 
@@ -345,6 +354,14 @@ static MetaBoolPreference preferences_bool[] =
       &resize_with_right_button,
       FALSE,
     },
+    {
+      { "edge-tiling",
+        SCHEMA_METACITY,
+        META_PREF_EDGE_TILING,
+      },
+      &edge_tiling,
+      FALSE,
+    },
     { { NULL, 0, 0 }, NULL, FALSE },
   };
 
@@ -628,7 +645,6 @@ handle_preference_update_int (GSettings *settings,
     }
 }
 
-
 /****************************************************************************/
 /* Listeners.                                                               */
 /****************************************************************************/
@@ -742,7 +758,6 @@ queue_changed (MetaPreference pref)
                                     changed_idle_handler, NULL, NULL);
 }
 
-
 /****************************************************************************/
 /* Initialisation.                                                          */
 /****************************************************************************/
@@ -793,7 +808,6 @@ meta_prefs_init (void)
   init_workspace_names ();
 }
 
-
 /****************************************************************************/
 /* Updates.                                                                 */
 /****************************************************************************/
@@ -927,7 +941,6 @@ meta_prefs_get_cursor_size (void)
   return cursor_size;
 }
 
-
 /****************************************************************************/
 /* Handlers for string preferences.                                         */
 /****************************************************************************/
@@ -1412,8 +1425,14 @@ meta_preference_to_string (MetaPreference pref)
     case META_PREF_RESIZE_WITH_RIGHT_BUTTON:
       return "RESIZE_WITH_RIGHT_BUTTON";
 
+    case META_PREF_EDGE_TILING:
+      return "EDGE_TILING";
+
     case META_PREF_FORCE_FULLSCREEN:
       return "FORCE_FULLSCREEN";
+
+    case META_PREF_PLACEMENT_MODE:
+      return "PLACEMENT_MODE";
     }
 
   return "(unknown)";
@@ -1739,6 +1758,12 @@ meta_prefs_get_gnome_animations ()
   return gnome_animations;
 }
 
+gboolean
+meta_prefs_get_edge_tiling ()
+{
+  return edge_tiling;
+}
+
 MetaKeyBindingAction
 meta_prefs_get_keybinding_action (const char *name)
 {
@@ -1824,6 +1849,12 @@ meta_prefs_get_force_fullscreen (void)
   return force_fullscreen;
 }
 
+MetaPlacementMode
+meta_prefs_get_placement_mode (void)
+{
+  return placement_mode;
+}
+
 void
 meta_prefs_set_compositing_manager (gboolean whether)
 {
@@ -1835,4 +1866,3 @@ meta_prefs_set_force_fullscreen (gboolean whether)
 {
   force_fullscreen = whether;
 }
-
