@@ -14,9 +14,7 @@
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 /**
@@ -129,6 +127,12 @@ keybind (switch-group-backward, handle_switch, META_TAB_LIST_GROUP,
 keybind (switch-windows, handle_switch, META_TAB_LIST_NORMAL, BINDING_REVERSES)
 keybind (switch-windows-backward, handle_switch, META_TAB_LIST_NORMAL,
          REVERSES_AND_REVERSED)
+/* We do not support switching applications, so we map these to switching
+ * windows */
+keybind (switch-applications, handle_switch, META_TAB_LIST_NORMAL,
+         BINDING_REVERSES)
+keybind (switch-applications-backward, handle_switch, META_TAB_LIST_NORMAL,
+         REVERSES_AND_REVERSED)
 keybind (switch-panels, handle_switch, META_TAB_LIST_DOCKS, BINDING_REVERSES)
 keybind (switch-panels-backward, handle_switch, META_TAB_LIST_DOCKS,
          REVERSES_AND_REVERSED)
