@@ -17,9 +17,7 @@
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef META_PREFS_H
@@ -60,9 +58,20 @@ typedef enum
   META_PREF_CURSOR_SIZE,
   META_PREF_COMPOSITING_MANAGER,
   META_PREF_RESIZE_WITH_RIGHT_BUTTON,
-  META_PREF_FORCE_FULLSCREEN
+  META_PREF_EDGE_TILING,
+  META_PREF_FORCE_FULLSCREEN,
+  META_PREF_PLACEMENT_MODE
 } MetaPreference;
 
+typedef enum
+{
+  META_PLACEMENT_MODE_SMART,
+  META_PLACEMENT_MODE_CASCADE,
+  META_PLACEMENT_MODE_CENTER,
+  META_PLACEMENT_MODE_ORIGIN,
+  META_PLACEMENT_MODE_RANDOM
+} MetaPlacementMode;
+
 typedef void (* MetaPrefsChangedFunc) (MetaPreference pref,
                                        gpointer       data);
 
@@ -91,6 +100,7 @@ int                         meta_prefs_get_auto_raise_delay   (void);
 gboolean                    meta_prefs_get_reduced_resources  (void);
 gboolean                    meta_prefs_get_gnome_accessibility (void);
 gboolean                    meta_prefs_get_gnome_animations   (void);
+gboolean                    meta_prefs_get_edge_tiling        (void);
 
 const char*                 meta_prefs_get_screenshot_command (void);
 
@@ -116,6 +126,8 @@ int         meta_prefs_get_cursor_size       (void);
 gboolean    meta_prefs_get_compositing_manager (void);
 gboolean    meta_prefs_get_force_fullscreen  (void);
 
+MetaPlacementMode meta_prefs_get_placement_mode (void);
+
 /**
  * Sets whether the compositor is turned on.
  *
@@ -149,6 +161,8 @@ typedef enum _MetaKeyBindingAction
   META_KEYBINDING_ACTION_WORKSPACE_RIGHT,
   META_KEYBINDING_ACTION_WORKSPACE_UP,
   META_KEYBINDING_ACTION_WORKSPACE_DOWN,
+  META_KEYBINDING_ACTION_SWITCH_APPLICATIONS,
+  META_KEYBINDING_ACTION_SWITCH_APPLICATIONS_BACKWARD,
   META_KEYBINDING_ACTION_SWITCH_GROUP,
   META_KEYBINDING_ACTION_SWITCH_GROUP_BACKWARD,
   META_KEYBINDING_ACTION_SWITCH_WINDOWS,
