@@ -17,9 +17,7 @@
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifndef META_UTIL_H
@@ -99,9 +97,9 @@ char* meta_g_utf8_strndup (const gchar *src, gsize n);
 void  meta_free_gslist_and_elements (GSList *list_to_deep_free);
 
 GPid meta_show_dialog (const char *type,
-                       const char *title,
                        const char *message,
-                       gint timeout,
+                       const char *timeout,
+                       const char *display,
                        const char *ok_text,
                        const char *cancel_text,
                        const int transient_for,
