@@ -14,9 +14,7 @@
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <gtk/gtk.h>
@@ -24,7 +22,10 @@
 #include <X11/Xatom.h>
 #include <unistd.h>
 
-static GtkWidget* do_appwindow (void);
+static void
+do_appwindow (GSimpleAction *action,
+              GVariant      *parameter,
+              gpointer       user_data);
 
 static gboolean aspect_on;
 
@@ -240,7 +241,7 @@ main (int argc, char **argv)
       g_error_free (err);
     }
   
-  do_appwindow ();
+  do_appwindow (NULL, NULL, NULL);
 
   gtk_main ();
   
@@ -307,15 +308,17 @@ response_cb (GtkDialog *dialog,
 }
 
 static void
-dialog_cb (GtkAction *action,
-           gpointer   callback_data)
+dialog_cb (GSimpleAction *action,
+           GVariant      *parameter,
+           gpointer       callback_data)
 {
   make_dialog (GTK_WIDGET (callback_data), 1);
 }
 
 static void
-modal_dialog_cb (GtkAction *action,
-                 gpointer   callback_data)
+modal_dialog_cb (GSimpleAction *action,
+                 GVariant      *parameter,
+                 gpointer       callback_data)
 {
   GtkWidget *dialog;
   
@@ -333,15 +336,17 @@ modal_dialog_cb (GtkAction *action,
 }
 
 static void
-no_parent_dialog_cb (GtkAction *action,
-                     gpointer   callback_data)
+no_parent_dialog_cb (GSimpleAction *action,
+                     GVariant      *parameter,
+                     gpointer       callback_data)
 {
   make_dialog (NULL, 1);
 }
 
 static void
-utility_cb (GtkAction *action,
-            gpointer   callback_data)
+utility_cb (GSimpleAction *action,
+            GVariant      *parameter,
+            gpointer       callback_data)
 {
   GtkWidget *window;
   GtkWidget *vbox;
@@ -353,7 +358,7 @@ utility_cb (GtkAction *action,
   
   gtk_window_set_transient_for (GTK_WINDOW (window), GTK_WINDOW (callback_data));
   
-  vbox = gtk_vbox_new (FALSE, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
   gtk_container_add (GTK_CONTAINER (window), vbox);
 
@@ -373,8 +378,9 @@ utility_cb (GtkAction *action,
 }
 
 static void
-toolbar_cb (GtkAction *action,
-            gpointer   callback_data)
+toolbar_cb (GSimpleAction *action,
+            GVariant      *parameter,
+            gpointer       callback_data)
 {
   GtkWidget *window;
   GtkWidget *vbox;
@@ -386,7 +392,7 @@ toolbar_cb (GtkAction *action,
   
   gtk_window_set_transient_for (GTK_WINDOW (window), GTK_WINDOW (callback_data));
   
-  vbox = gtk_vbox_new (FALSE, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
   gtk_container_add (GTK_CONTAINER (window), vbox);
 
@@ -397,8 +403,9 @@ toolbar_cb (GtkAction *action,
 }
 
 static void
-menu_cb (GtkAction *action,
-         gpointer   callback_data)
+menu_cb (GSimpleAction *action,
+         GVariant      *parameter,
+         gpointer       callback_data)
 {
   GtkWidget *window;
   GtkWidget *vbox;
@@ -410,7 +417,7 @@ menu_cb (GtkAction *action,
   
   gtk_window_set_transient_for (GTK_WINDOW (window), GTK_WINDOW (callback_data));
   
-  vbox = gtk_vbox_new (FALSE, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
   gtk_container_add (GTK_CONTAINER (window), vbox);
 
@@ -421,8 +428,9 @@ menu_cb (GtkAction *action,
 }
 
 static void
-override_redirect_cb (GtkAction *action,
-                      gpointer   callback_data)
+override_redirect_cb (GSimpleAction *action,
+                      GVariant      *parameter,
+                      gpointer       callback_data)
 {
   GtkWidget *window;
   GtkWidget *vbox;
@@ -431,7 +439,7 @@ override_redirect_cb (GtkAction *action,
   window = gtk_window_new (GTK_WINDOW_POPUP);
   gtk_window_set_title (GTK_WINDOW (window), "Override Redirect");
   
-  vbox = gtk_vbox_new (FALSE, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
   gtk_container_add (GTK_CONTAINER (window), vbox);
 
@@ -442,8 +450,9 @@ override_redirect_cb (GtkAction *action,
 }
 
 static void
-border_only_cb (GtkAction *action,
-                gpointer   callback_data)
+border_only_cb (GSimpleAction *action,
+                GVariant      *parameter,
+                gpointer       callback_data)
 {
   GtkWidget *window;
   GtkWidget *vbox;
@@ -455,7 +464,7 @@ border_only_cb (GtkAction *action,
   
   gtk_window_set_transient_for (GTK_WINDOW (window), GTK_WINDOW (callback_data));
   
-  vbox = gtk_vbox_new (FALSE, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 
   gtk_container_add (GTK_CONTAINER (window), vbox);
 
@@ -465,29 +474,6 @@ border_only_cb (GtkAction *action,
   gtk_widget_show_all (window);
 }
 
-#if 0
-static void
-changing_icon_cb (GtkAction *action,
-                  gpointer             callback_data)
-{
-  GtkWidget *window;
-  GtkWidget *vbox;
-  GtkWidget *label;
-  
-  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
-  gtk_window_set_title (GTK_WINDOW (window), "Changing Icon");
-  
-  vbox = gtk_vbox_new (FALSE, 0);
-
-  gtk_container_add (GTK_CONTAINER (window), vbox);
-
-  label = gtk_label_new ("This window has an icon that changes over time");
-  gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
-  
-  gtk_widget_show_all (window);
-}
-#endif
-
 static gboolean
 focus_in_event_cb (GtkWidget *window,
                    GdkEvent  *event,
@@ -534,8 +520,9 @@ focus_label (GtkWidget *window)
 }
 
 static void
-splashscreen_cb (GtkAction *action,
-                 gpointer   callback_data)
+splashscreen_cb (GSimpleAction *action,
+                 GVariant      *parameter,
+                 gpointer       callback_data)
 {
   GtkWidget *window;
   GtkWidget *image;
@@ -545,9 +532,9 @@ splashscreen_cb (GtkAction *action,
   set_gtk_window_type (GTK_WINDOW (window), "_NET_WM_WINDOW_TYPE_SPLASHSCREEN");
   gtk_window_set_title (GTK_WINDOW (window), "Splashscreen");
   
-  vbox = gtk_vbox_new (FALSE, 0);
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
   
-  image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
+  image = gtk_image_new_from_icon_name ("dialog-information", GTK_ICON_SIZE_DIALOG);
   gtk_box_pack_start (GTK_BOX (vbox), image, FALSE, FALSE, 0);
 
   gtk_box_pack_start (GTK_BOX (vbox), focus_label (window), FALSE, FALSE, 0);  
@@ -581,11 +568,11 @@ make_dock (int type)
     {
     case DOCK_LEFT:
     case DOCK_RIGHT:      
-      box = gtk_vbox_new (FALSE, 0);
+      box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
       break;
     case DOCK_TOP:
     case DOCK_BOTTOM:
-      box = gtk_hbox_new (FALSE, 0);
+      box = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
       break;
     case DOCK_ALL:
       break;
@@ -594,7 +581,7 @@ make_dock (int type)
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   set_gtk_window_type (GTK_WINDOW (window), "_NET_WM_WINDOW_TYPE_DOCK");
   
-  image = gtk_image_new_from_stock (GTK_STOCK_DIALOG_INFO, GTK_ICON_SIZE_DIALOG);
+  image = gtk_image_new_from_icon_name ("dialog-information", GTK_ICON_SIZE_DIALOG);
   gtk_box_pack_start (GTK_BOX (box), image, FALSE, FALSE, 0);  
   
   gtk_box_pack_start (GTK_BOX (box), focus_label (window), FALSE, FALSE, 0);  
@@ -642,23 +629,24 @@ make_dock (int type)
 }
 
 static void
-dock_cb (GtkAction *action,
-         gpointer   callback_data)
+dock_cb (GSimpleAction *action,
+         GVariant      *parameter,
+         gpointer       callback_data)
 {
   guint callback_action;
   const gchar *name;
 
-  name = gtk_action_get_name (action);
+  g_object_get (G_OBJECT (action), "name", &name, NULL);
 
-  if (!g_strcmp0 (name, "Top dock"))
+  if (!g_strcmp0 (name, "top-dock"))
     callback_action = DOCK_TOP;
-  else if (!g_strcmp0 (name, "Bottom dock"))
+  else if (!g_strcmp0 (name, "bottom-dock"))
     callback_action = DOCK_BOTTOM;
-  else if (!g_strcmp0 (name, "Left dock"))
+  else if (!g_strcmp0 (name, "left-dock"))
     callback_action = DOCK_LEFT;
-  else if (!g_strcmp0 (name, "Right dock"))
+  else if (!g_strcmp0 (name, "right-dock"))
     callback_action = DOCK_RIGHT;
-  else if (!g_strcmp0 (name, "All docks"))
+  else if (!g_strcmp0 (name, "all-docks"))
     callback_action = DOCK_ALL;
   else
     return;
@@ -677,12 +665,13 @@ dock_cb (GtkAction *action,
 }
 
 static void
-desktop_cb (GtkAction *action,
-            gpointer   callback_data)
+desktop_cb (GSimpleAction *action,
+            GVariant      *parameter,
+            gpointer       callback_data)
 {
   GtkWidget *window;
   GtkWidget *label;
-  GdkColor desktop_color;
+  GdkRGBA    desktop_color;
   
   window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
   set_gtk_window_type (GTK_WINDOW (window), "_NET_WM_WINDOW_TYPE_DESKTOP");
@@ -691,11 +680,12 @@ desktop_cb (GtkAction *action,
                                gdk_screen_width (), gdk_screen_height ());
   gtk_window_move (GTK_WINDOW (window), 0, 0);
   
-  desktop_color.red = 0x5144;
-  desktop_color.green = 0x75D6;
-  desktop_color.blue = 0xA699;
+  desktop_color.red = 0.32;
+  desktop_color.green = 0.46;
+  desktop_color.blue = 0.65;
+  desktop_color.alpha = 1.0;
 
-  gtk_widget_modify_bg (window, GTK_STATE_NORMAL, &desktop_color);
+  gtk_widget_override_background_color (window, 0, &desktop_color);
   
   label = focus_label (window);
   
@@ -705,15 +695,17 @@ desktop_cb (GtkAction *action,
 }
 
 static void
-sleep_cb (GtkAction *action,
-          gpointer   data)
+sleep_cb (GSimpleAction *action,
+          GVariant      *parameter,
+          gpointer       data)
 {
   sleep (1000);
 }
 
 static void
-toggle_aspect_ratio (GtkAction *action,
-                     gpointer   data)
+toggle_aspect_ratio (GSimpleAction *action,
+                     GVariant      *parameter,
+                     gpointer       data)
 {
   GtkWidget *window;
   GdkGeometry geom;
@@ -742,8 +734,9 @@ toggle_aspect_ratio (GtkAction *action,
 }
 
 static void
-toggle_decorated_cb (GtkWidget *button,
-                     gpointer   data)
+toggle_decorated_cb (GSimpleAction *action,
+                     GVariant      *parameter,
+                     gpointer       data)
 {
   GtkWidget *window;
   window = gtk_widget_get_ancestor (data, GTK_TYPE_WINDOW);
@@ -753,8 +746,9 @@ toggle_decorated_cb (GtkWidget *button,
 }
 
 static void
-clicked_toolbar_cb (GtkWidget *button,
-                    gpointer   data)
+clicked_toolbar_cb (GSimpleAction *action,
+                    GVariant      *parameter,
+                    gpointer       data)
 {
   GtkWidget *dialog;
   
@@ -820,104 +814,156 @@ destroy_cb (GtkWidget *w, gpointer data)
     gtk_main_quit ();
 }
 
-static const gchar *menu_item_string =
-  "<ui>\n"
-    "<menubar>\n"
-      "<menu name='Windows' action='Windows'>\n"
-        "<menuitem name='Dialog' action='Dialog'/>\n"
-        "<menuitem name='Modal dialog' action='Modal dialog'/>\n"
-        "<menuitem name='Parentless dialog' action='Parentless dialog'/>\n"
-        "<menuitem name='Utility' action='Utility'/>\n"
-        "<menuitem name='Splashscreen' action='Splashscreen'/>\n"
-        "<menuitem name='Top dock' action='Top dock'/>\n"
-        "<menuitem name='Bottom dock' action='Bottom dock'/>\n"
-        "<menuitem name='Left dock' action='Left dock'/>\n"
-        "<menuitem name='Right dock' action='Right dock'/>\n"
-        "<menuitem name='All docks' action='All docks'/>\n"
-        "<menuitem name='Desktop' action='Desktop'/>\n"
-        "<menuitem name='Menu' action='Menu'/>\n"
-        "<menuitem name='Toolbar' action='Toolbar'/>\n"
-        "<menuitem name='Override Redirect' action='Override Redirect'/>\n"
-        "<menuitem name='Border Only' action='Border Only'/>\n"
-      "</menu>\n"
-    "</menubar>\n"
-    "<toolbar>\n"
-      "<toolitem name='New' action='New'/>\n"
-      "<toolitem name='Lock' action='Lock'/>\n"
-      "<toolitem name='Decorations' action='Decorations'/>\n"
-      "<toolitem name='Ratio' action='Ratio'/>\n"
-      "<toolitem name='Quit' action='Quit'/>\n"
-    "</toolbar>\n"
-  "</ui>\n";
+static const gchar *xml =
+  "<interface>"
+    "<menu id='menubar'>"
+      "<submenu>"
+        "<attribute name='label'>Windows</attribute>"
+        "<section>"
+          "<item>"
+            "<attribute name='label'>Dialog</attribute>"
+            "<attribute name='action'>demo.dialog1</attribute>"
+            "<attribute name='accel'>&lt;control&gt;d</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Modal dialog</attribute>"
+            "<attribute name='action'>demo.dialog2</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Parentless dialog</attribute>"
+            "<attribute name='action'>demo.dialog3</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Utility</attribute>"
+            "<attribute name='action'>demo.utility</attribute>"
+            "<attribute name='accel'>&lt;control&gt;u</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Splashscreen</attribute>"
+            "<attribute name='action'>demo.splashscreen</attribute>"
+            "<attribute name='accel'>&lt;control&gt;s</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Top dock</attribute>"
+            "<attribute name='action'>demo.top-dock</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Bottom dock</attribute>"
+            "<attribute name='action'>demo.bottom-dock</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Left dock</attribute>"
+            "<attribute name='action'>demo.left-dock</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Right dock</attribute>"
+            "<attribute name='action'>demo.right-dock</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>All docks</attribute>"
+            "<attribute name='action'>demo.all-docks</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Desktop</attribute>"
+            "<attribute name='action'>demo.desktop</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Menu</attribute>"
+            "<attribute name='action'>demo.menu</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Toolbar</attribute>"
+            "<attribute name='action'>demo.toolbar</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Override Redirect</attribute>"
+            "<attribute name='action'>demo.override-redirect</attribute>"
+          "</item>"
+          "<item>"
+            "<attribute name='label'>Border Only</attribute>"
+            "<attribute name='action'>demo.border-only</attribute>"
+          "</item>"
+        "</section>"
+      "</submenu>"
+    "</menu>"
+  "</interface>";
 
-static const GtkActionEntry menu_items[] =
+static GActionEntry demo_entries[] =
 {
-  { "Windows",                  NULL,   "_Windows",             NULL,
-    NULL,       NULL },
-  { "Dialog",                   NULL,   "_Dialog",              "<control>d",
-    NULL,       G_CALLBACK (dialog_cb) },
-  { "Modal dialog",             NULL,   "_Modal dialog",        NULL,
-    NULL,       G_CALLBACK (modal_dialog_cb) },
-  { "Parentless dialog",        NULL,   "_Parentless dialog",   NULL,
-    NULL,       G_CALLBACK (no_parent_dialog_cb) },
-  { "Utility",                  NULL,   "_Utility",             "<control>u",
-    NULL,       G_CALLBACK (utility_cb) },
-  { "Splashscreen",             NULL,   "_Splashscreen",        "<control>s",
-    NULL,       G_CALLBACK (splashscreen_cb) },
-  { "Top dock",                 NULL,   "_Top dock",            NULL,
-    NULL,       G_CALLBACK (dock_cb) },
-  { "Bottom dock",              NULL,   "_Bottom dock",         NULL,
-    NULL,       G_CALLBACK (dock_cb) },
-  { "Left dock",                NULL,   "_Left dock",           NULL,
-    NULL,       G_CALLBACK (dock_cb) },
-  { "Right dock",               NULL,   "_Right dock",          NULL,
-    NULL,       G_CALLBACK (dock_cb) },
-  { "All docks",                NULL,   "_All docks",           NULL,
-    NULL,       G_CALLBACK (dock_cb) },
-  { "Desktop",                  NULL,   "Des_ktop",             NULL,
-    NULL,       G_CALLBACK (desktop_cb) },
-  { "Menu",                     NULL,   "Me_nu",                NULL,
-    NULL,       G_CALLBACK (menu_cb) },
-  { "Toolbar",                  NULL,   "Tool_bar",             NULL,
-    NULL,       G_CALLBACK (toolbar_cb) },
-  { "Override Redirect",        NULL,   "Override Redirect",    NULL,
-    NULL,       G_CALLBACK (override_redirect_cb) },
-  { "Border Only",              NULL,   "Border Only",          NULL,
-    NULL,       G_CALLBACK (border_only_cb) }
-};
-
-static const GtkActionEntry tool_items[] =
-{
-  { "New",              GTK_STOCK_NEW,  NULL,   NULL,
-    "Open another one of these windows", G_CALLBACK (do_appwindow) },
-  { "Lock",             GTK_STOCK_OPEN, NULL,   NULL,
-    "This is a demo button that"
-    " locks up the demo",                G_CALLBACK (sleep_cb) },
-  { "Decorations",      GTK_STOCK_OPEN, NULL,   NULL,
-    "This is a demo button that "
-    "toggles window decorations",        G_CALLBACK (toggle_decorated_cb) },
-  { "Quit",             GTK_STOCK_QUIT, NULL,   NULL,
-    "This is a demo button with "
-    " a 'quit' icon",                    G_CALLBACK (clicked_toolbar_cb) },
-  { "Ratio",            GTK_STOCK_OPEN, NULL,   NULL,
-    "This is a demo button that locks the aspect ratio "
-    "using a hint",                     G_CALLBACK (toggle_aspect_ratio) }
+  /* menubar */
+  { "dialog1",           dialog_cb,            NULL, NULL, NULL, {} },
+  { "dialog2",           modal_dialog_cb,      NULL, NULL, NULL, {} },
+  { "dialog3",           no_parent_dialog_cb,  NULL, NULL, NULL, {} },
+  { "utility",           utility_cb,           NULL, NULL, NULL, {} },
+  { "splashscreen",      splashscreen_cb,      NULL, NULL, NULL, {} },
+  { "top-dock",          dock_cb,              NULL, NULL, NULL, {} },
+  { "bottom-dock",       dock_cb,              NULL, NULL, NULL, {} },
+  { "left-dock",         dock_cb,              NULL, NULL, NULL, {} },
+  { "right-dock",        dock_cb,              NULL, NULL, NULL, {} },
+  { "all-docks",         dock_cb,              NULL, NULL, NULL, {} },
+  { "desktop",           desktop_cb,           NULL, NULL, NULL, {} },
+  { "menu",              menu_cb,              NULL, NULL, NULL, {} },
+  { "toolbar",           toolbar_cb,           NULL, NULL, NULL, {} },
+  { "override-redirect", override_redirect_cb, NULL, NULL, NULL, {} },
+  { "border-only",       border_only_cb,       NULL, NULL, NULL, {} },
+  /* toolbar */
+  { "new",               do_appwindow,         NULL, NULL, NULL, {} },
+  { "lock",              sleep_cb,             NULL, NULL, NULL, {} },
+  { "decorations",       toggle_decorated_cb,  NULL, NULL, NULL, {} },
+  { "quit",              clicked_toolbar_cb,   NULL, NULL, NULL, {} },
+  { "ratio",             toggle_aspect_ratio,  NULL, NULL, NULL, {} },
 };
 
 static GtkWidget *
-do_appwindow (void)
+create_toolbar (void)
+{
+  GtkWidget *toolbar;
+  GtkToolItem *item;
+
+  toolbar = gtk_toolbar_new ();
+
+  item = gtk_tool_button_new (gtk_image_new_from_icon_name ("document-new", GTK_ICON_SIZE_SMALL_TOOLBAR), NULL);
+  gtk_tool_item_set_tooltip_markup (item, "Open another one of these windows");
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (item), "demo.new");
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+
+  item = gtk_tool_button_new (gtk_image_new_from_icon_name ("document-open", GTK_ICON_SIZE_SMALL_TOOLBAR), NULL);
+  gtk_tool_item_set_tooltip_markup (item, "This is a demo button that locks up the demo");
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (item), "demo.lock");
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+
+  item = gtk_tool_button_new (gtk_image_new_from_icon_name ("document-open", GTK_ICON_SIZE_SMALL_TOOLBAR), NULL);
+  gtk_tool_item_set_tooltip_markup (item, "This is a demo button that toggles window decorations");
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (item), "demo.decorations");
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+
+  item = gtk_tool_button_new (gtk_image_new_from_icon_name ("document-open", GTK_ICON_SIZE_SMALL_TOOLBAR), NULL);
+  gtk_tool_item_set_tooltip_markup (item, "This is a demo button that locks the aspect ratio using a hint");
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (item), "demo.ratio");
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+
+  item = gtk_tool_button_new (gtk_image_new_from_icon_name ("gtk-quit", GTK_ICON_SIZE_SMALL_TOOLBAR), NULL);
+  gtk_tool_item_set_tooltip_markup (item, "This is a demo button with a 'quit' icon");
+  gtk_actionable_set_action_name (GTK_ACTIONABLE (item), "demo.quit");
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), item, -1);
+
+  return toolbar;
+}
+
+static void
+do_appwindow (GSimpleAction *action,
+              GVariant      *parameter,
+              gpointer       user_data)
 {
   GtkWidget *window;
-  GtkWidget *table;
-  GtkWidget *handlebox;
+  GtkWidget *grid;
   GtkWidget *statusbar;
   GtkWidget *contents;
   GtkWidget *sw;
   GtkTextBuffer *buffer;
-  GtkActionGroup *action_group;
-  GtkUIManager *ui_manager;
+  GSimpleActionGroup *action_group;
+  GtkBuilder *builder;
 
-      
   /* Create the toplevel window
    */
 
@@ -931,43 +977,42 @@ do_appwindow (void)
   g_signal_connect (G_OBJECT (window), "destroy",
                     G_CALLBACK (destroy_cb), NULL);
       
-  table = gtk_table_new (1, 4, FALSE);
-      
-  gtk_container_add (GTK_CONTAINER (window), table);
-      
+  grid = gtk_grid_new ();
+
+  gtk_widget_set_vexpand (grid, TRUE);
+  gtk_widget_set_hexpand (grid, TRUE);
+ 
+  gtk_container_add (GTK_CONTAINER (window), grid);
+
+  action_group = g_simple_action_group_new ();
+  builder = gtk_builder_new_from_string (xml, -1);
+
+  g_action_map_add_action_entries (G_ACTION_MAP (action_group),
+                                   demo_entries,
+                                   G_N_ELEMENTS (demo_entries),
+                                   window);
+  gtk_widget_insert_action_group (window, "demo", G_ACTION_GROUP (action_group));
+
   /* Create the menubar
    */
-      
-  contents = gtk_text_view_new ();
 
-  action_group = gtk_action_group_new ("mainmenu");
-  gtk_action_group_add_actions (action_group,
-                                menu_items,
-                                G_N_ELEMENTS (menu_items),
-                                window);
-  gtk_action_group_add_actions (action_group,
-                                tool_items,
-                                G_N_ELEMENTS (tool_items),
-                                window);
+  GMenuModel *model = G_MENU_MODEL (gtk_builder_get_object (builder, "menubar"));
+  GtkWidget *menubar = gtk_menu_bar_new_from_model (model);
+  gtk_grid_attach (GTK_GRID (grid), menubar, 0, 0, 1, 1);
+  gtk_widget_set_hexpand (menubar, TRUE);
 
-  ui_manager = gtk_ui_manager_new ();
+  /* Create the toolbar
+   */
 
-  gtk_ui_manager_insert_action_group (ui_manager, action_group, 0);
-
-  /* create menu items */
-
-  gtk_ui_manager_add_ui_from_string (ui_manager, menu_item_string, -1, NULL);
-
-  gtk_table_attach (GTK_TABLE (table),
-                    gtk_ui_manager_get_widget (ui_manager, "/ui/menubar"),
-                    /* X direction */          /* Y direction */
-                    0, 1,                      0, 1,
-                    GTK_EXPAND | GTK_FILL,     0,
-                    0,                         0);
+  GtkWidget *toolbar = create_toolbar ();
+  gtk_grid_attach (GTK_GRID (grid), toolbar, 0, 1, 1, 1);
+  gtk_widget_set_hexpand (toolbar, TRUE);
 
   /* Create document
    */
 
+  contents = gtk_text_view_new ();
+
   sw = gtk_scrolled_window_new (NULL, NULL);
 
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
@@ -976,49 +1021,30 @@ do_appwindow (void)
 
   gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
                                        GTK_SHADOW_IN);
-      
-  gtk_table_attach (GTK_TABLE (table),
-                    sw,
-                    /* X direction */       /* Y direction */
-                    0, 1,                   2, 3,
-                    GTK_EXPAND | GTK_FILL,  GTK_EXPAND | GTK_FILL,
-                    0,                      0);
+
+  gtk_grid_attach (GTK_GRID (grid), sw, 0, 2, 1, 1);
+
+  gtk_widget_set_hexpand (sw, TRUE);
+  gtk_widget_set_vexpand (sw, TRUE);
 
   gtk_window_set_default_size (GTK_WINDOW (window),
                                200, 200);
-      
+
   gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (contents),
                                PANGO_WRAP_WORD);
-      
+
   gtk_container_add (GTK_CONTAINER (sw),
                      contents);
 
-  /* Create the toolbar
+  /* Create statusbar
    */
 
-  handlebox = gtk_handle_box_new ();
-
-  gtk_container_add (GTK_CONTAINER (handlebox),
-                     gtk_ui_manager_get_widget (ui_manager, "/ui/toolbar"));
-      
-  gtk_table_attach (GTK_TABLE (table),
-                    handlebox,
-                    /* X direction */       /* Y direction */
-                    0, 1,                   1, 2,
-                    GTK_EXPAND | GTK_FILL,  0,
-                    0,                      0);
-
-  /* Create statusbar */
-
   statusbar = gtk_statusbar_new ();
-  gtk_table_attach (GTK_TABLE (table),
-                    statusbar,
-                    /* X direction */       /* Y direction */
-                    0, 1,                   3, 4,
-                    GTK_EXPAND | GTK_FILL,  0,
-                    0,                      0);
+  gtk_grid_attach (GTK_GRID (grid), statusbar, 0, 3, 1, 1);
+  gtk_widget_set_hexpand (statusbar, TRUE);
 
   /* Show text widget info in the statusbar */
+
   buffer = gtk_text_view_get_buffer (GTK_TEXT_VIEW (contents));
 
   gtk_text_buffer_set_text (buffer,
@@ -1027,7 +1053,7 @@ do_appwindow (void)
                             "Be sure to tear off the menu and toolbar, those are also "
                             "a special kind of window.",
                             -1);
-      
+
   g_signal_connect_object (buffer,
                            "changed",
                            G_CALLBACK (update_statusbar),
@@ -1039,14 +1065,11 @@ do_appwindow (void)
                            G_CALLBACK (mark_set_callback),
                            statusbar,
                            0);
-      
+
   update_statusbar (buffer, GTK_STATUSBAR (statusbar));
 
   gtk_widget_show_all (window);
 
-  g_object_unref (ui_manager);
-
-  return window;
+  g_object_unref (action_group);
+  g_object_unref (builder);
 }
-
-
