@@ -18,9 +18,7 @@
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -35,6 +33,7 @@
 #include "draw-workspace.h"
 #include <gtk/gtk.h>
 #include <math.h>
+#include <X11/Xatom.h>
 
 #define OUTSIDE_SELECT_RECT 2
 #define INSIDE_SELECT_RECT 2
@@ -72,15 +71,12 @@ static void       select_workspace         (GtkWidget *widget);
 static void       unselect_workspace       (GtkWidget *widget);
 
 static gboolean
-outline_window_expose (GtkWidget      *widget,
-                       GdkEventExpose *event,
-                       gpointer        data)
+outline_window_draw (GtkWidget *widget,
+                     cairo_t   *cr,
+                     gpointer   data)
 {
   MetaTabPopup *popup;
   TabEntry *te;  
-  GtkStyle *style;
-  GdkWindow *window;
-  cairo_t *cr;
   
   popup = data;
 
@@ -88,12 +84,9 @@ outline_window_expose (GtkWidget      *widget,
     return FALSE;
 
   te = popup->current_selected_entry;
-  window = gtk_widget_get_window (widget);
-  style = gtk_widget_get_style (widget);
-  cr = gdk_cairo_create (window);
   
   cairo_set_line_width (cr, 1.0);
-  gdk_cairo_set_source_color (cr, &style->white);
+  cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
 
   cairo_rectangle (cr,
                    0.5, 0.5,
@@ -107,8 +100,6 @@ outline_window_expose (GtkWidget      *widget,
                    te->inner_rect.height + 1);
   cairo_stroke (cr);
 
-  cairo_destroy (cr);
-
   return FALSE;
 }
 
@@ -227,7 +218,7 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   MetaTabPopup *popup;
   int i, left, right, top, bottom;
   int height;
-  GtkWidget *table;
+  GtkWidget *grid;
   GtkWidget *vbox;
   GtkWidget *align;
   GList *tmp;
@@ -239,19 +230,47 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   
   popup = g_new (MetaTabPopup, 1);
 
-  popup->outline_window = gtk_window_new (GTK_WINDOW_POPUP);
-
   screen = gdk_display_get_screen (gdk_display_get_default (),
                                    screen_number);
-  gtk_window_set_screen (GTK_WINDOW (popup->outline_window),
-                         screen);
 
-  gtk_widget_set_app_paintable (popup->outline_window, TRUE);
-  gtk_widget_realize (popup->outline_window);
+  if (outline)
+    {
+      GdkRGBA black = { 0.0, 0.0, 0.0, 1.0 };
+      unsigned long data[1];
+      GdkWindow *window;
+      Display *xdisplay;
+      Window xwindow;
+
+      popup->outline_window = gtk_window_new (GTK_WINDOW_POPUP);
+
+      gtk_window_set_screen (GTK_WINDOW (popup->outline_window),
+                             screen);
+
+      gtk_widget_set_app_paintable (popup->outline_window, TRUE);
+      gtk_widget_realize (popup->outline_window);
+
+      window = gtk_widget_get_window (popup->outline_window);
+      xdisplay = GDK_WINDOW_XDISPLAY (window);
+      xwindow = GDK_WINDOW_XID (window);
+      data[0] = 0;
+
+      gdk_error_trap_push ();
+      XChangeProperty (xdisplay, xwindow, XInternAtom (xdisplay, "METACITY_WINDOW_HAVE_SHADOW", False),
+                       XA_CARDINAL, 32, PropModeReplace,
+                       (guchar *) data, 1);
+      gdk_error_trap_pop_ignored ();
+
+      gdk_window_set_background_rgba (gtk_widget_get_window (popup->outline_window),
+                                      &black);
+
+      g_signal_connect (G_OBJECT (popup->outline_window), "draw",
+                        G_CALLBACK (outline_window_draw), popup);
+
+      gtk_widget_show (popup->outline_window);
+    }
+  else
+    popup->outline_window = NULL;
 
-  g_signal_connect (G_OBJECT (popup->outline_window), "expose_event",
-                    G_CALLBACK (outline_window_expose), popup);
-  
   popup->window = gtk_window_new (GTK_WINDOW_POPUP);
 
   gtk_window_set_screen (GTK_WINDOW (popup->window),
@@ -281,12 +300,12 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   if (i % width)
     height += 1;
 
-  table = gtk_table_new (height, width, FALSE);
-  vbox = gtk_vbox_new (FALSE, 0);
+  grid = gtk_grid_new ();
+  vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
   
   frame = gtk_frame_new (NULL);
   gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);
-  gtk_container_set_border_width (GTK_CONTAINER (table), 1);
+  gtk_container_set_border_width (GTK_CONTAINER (grid), 1);
   gtk_container_add (GTK_CONTAINER (popup->window),
                      frame);
   gtk_container_add (GTK_CONTAINER (frame),
@@ -297,7 +316,7 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
   gtk_box_pack_start (GTK_BOX (vbox), align, TRUE, TRUE, 0);
 
   gtk_container_add (GTK_CONTAINER (align),
-                     table);
+                     grid);
 
   popup->label = gtk_label_new ("");
 
@@ -359,16 +378,12 @@ meta_ui_tab_popup_new (const MetaTabEntry *entries,
 
           te->widget = image;
 
-          gtk_table_attach (GTK_TABLE (table),
-                            te->widget,
-                            left, right,           top, bottom,
-                            0,                     0,
-                            0,                     0);
+          gtk_grid_attach (GTK_GRID (grid), te->widget, left, top, 1, 1);
 
           /* Efficiency rules! */
           gtk_label_set_markup (GTK_LABEL (popup->label),
                               te->title);
-          gtk_widget_size_request (popup->label, &req);
+          gtk_widget_get_preferred_size (popup->label, &req, NULL);
           max_label_width = MAX (max_label_width, req.width);
           
           tmp = tmp->next;
@@ -421,8 +436,9 @@ void
 meta_ui_tab_popup_free (MetaTabPopup *popup)
 {
   meta_verbose ("Destroying tab popup window\n");
-  
-  gtk_widget_destroy (popup->outline_window);
+
+  if (popup->outline_window != NULL)
+    gtk_widget_destroy (popup->outline_window);
   gtk_widget_destroy (popup->window);
   
   g_list_foreach (popup->entries, free_tab_entry, NULL);
@@ -455,12 +471,6 @@ static void
 display_entry (MetaTabPopup *popup,
                TabEntry     *te)
 {
-  GdkRectangle rect;
-  GdkRegion *region;
-  GdkRegion *inner_region;
-  GdkWindow *window;
-
-  
   if (popup->current_selected_entry)
   {
     if (popup->outline)
@@ -478,40 +488,32 @@ display_entry (MetaTabPopup *popup,
 
   if (popup->outline)
     {
+      GdkRectangle rect;
+      GdkWindow *window;
+      cairo_region_t *region;
+
       window = gtk_widget_get_window (popup->outline_window);
 
       /* Do stuff behind gtk's back */
       gdk_window_hide (window);
       meta_core_increment_event_serial (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
-  
+
       rect = te->rect;
       rect.x = 0;
       rect.y = 0;
 
-      gdk_window_move_resize (window,
-                              te->rect.x, te->rect.y,
-                              te->rect.width, te->rect.height);
-  
-      gdk_window_set_background (window,
-                                 &gtk_widget_get_style (popup->outline_window)->black);
-  
-      region = gdk_region_rectangle (&rect);
-      inner_region = gdk_region_rectangle (&te->inner_rect);
-      gdk_region_subtract (region, inner_region);
-      gdk_region_destroy (inner_region);
-  
-      gdk_window_shape_combine_region (window,
+      gtk_window_move (GTK_WINDOW (popup->outline_window), te->rect.x, te->rect.y);
+      gtk_window_resize (GTK_WINDOW (popup->outline_window), te->rect.width, te->rect.height);
+
+      region = cairo_region_create_rectangle (&rect);
+      cairo_region_subtract_rectangle (region, &te->inner_rect);
+
+      gdk_window_shape_combine_region (gtk_widget_get_window (popup->outline_window),
                                        region,
                                        0, 0);
 
-      gdk_region_destroy (region);
-  
-      /* This should piss off gtk a bit, but we don't want to raise
-       * above the tab popup.  So, instead of calling gtk_widget_show,
-       * we manually set the window as mapped and then manually map it
-       * with gdk functions.
-       */
-      gtk_widget_set_mapped (popup->outline_window, TRUE);
+      cairo_region_destroy (region);
+
       gdk_window_show_unraised (window);
     }
 
@@ -648,8 +650,8 @@ unselect_image (GtkWidget *widget)
 }
 
 static void     meta_select_image_class_init   (MetaSelectImageClass *klass);
-static gboolean meta_select_image_expose_event (GtkWidget            *widget,
-                                                GdkEventExpose       *event);
+static gboolean meta_select_image_draw         (GtkWidget            *widget,
+                                                cairo_t              *cr);
 
 static GtkImageClass *parent_class;
 
@@ -688,74 +690,59 @@ meta_select_image_class_init (MetaSelectImageClass *klass)
 
   widget_class = GTK_WIDGET_CLASS (klass);
   
-  widget_class->expose_event = meta_select_image_expose_event;
+  widget_class->draw= meta_select_image_draw;
 }
 
 static gboolean
-meta_select_image_expose_event (GtkWidget      *widget,
-                                GdkEventExpose *event)
+meta_select_image_draw (GtkWidget *widget,
+                        cairo_t   *cr)
 {
+  GtkAllocation allocation;
+
+  gtk_widget_get_allocation (widget, &allocation);
+
   if (META_SELECT_IMAGE (widget)->selected)
     {
-      GtkAllocation allocation;
-      GtkRequisition requisition;
       GtkMisc *misc;
-      GtkStyle *style;
-      GtkStateType state;
-      GdkWindow *window;
+      GtkRequisition requisition;
+      GtkStyleContext *context;
+      GdkRGBA color;
       int x, y, w, h;
-      int xpad, ypad;
-      float xalign, yalign;
-      cairo_t *cr;
+      gint xpad, ypad;
+      gfloat xalign, yalign;
 
       misc = GTK_MISC (widget);
       
-      gtk_widget_get_allocation (widget, &allocation);
-      gtk_widget_get_requisition (widget, &requisition);
-      gtk_misc_get_padding (misc, &xpad, &ypad);
+      gtk_widget_get_preferred_size (widget, &requisition, 0);
       gtk_misc_get_alignment (misc, &xalign, &yalign);
+      gtk_misc_get_padding (misc, &xpad, &ypad);
 
-      x = (allocation.x * (1.0 - xalign) +
-           (allocation.x + allocation.width
-            - (requisition.width - xpad * 2)) * xalign) + 0.5;
-      y = (allocation.y * (1.0 - yalign) +
-           (allocation.y + allocation.height
-            - (requisition.height - ypad * 2)) * yalign) + 0.5;
+      x = (allocation.width - (requisition.width - xpad * 2)) * xalign + 0.5;
+      y = (allocation.height - (requisition.height - ypad * 2)) * yalign + 0.5;
 
       x -= INSIDE_SELECT_RECT + 1;
-      y -= INSIDE_SELECT_RECT + 1;      
+      y -= INSIDE_SELECT_RECT + 1;       
       
       w = requisition.width - OUTSIDE_SELECT_RECT * 2 - 1;
       h = requisition.height - OUTSIDE_SELECT_RECT * 2 - 1;
 
-      window = gtk_widget_get_window (widget);
-      style = gtk_widget_get_style (widget);
-      state = gtk_widget_get_state (widget);
-      cr = gdk_cairo_create (window);
+      context = gtk_widget_get_style_context (widget);
+
+      gtk_style_context_set_state (context,
+                                   gtk_widget_get_state_flags (widget));
+
+      gtk_style_context_lookup_color (context, "color", &color);
 
       cairo_set_line_width (cr, 2.0);
-      gdk_cairo_set_source_color (cr, &style->fg[state]);
+      cairo_set_source_rgb (cr, color.red, color.green, color.blue);
 
       cairo_rectangle (cr, x, y, w + 1, h + 1);
       cairo_stroke (cr);
 
       cairo_set_line_width (cr, 1.0);
-#if 0
-      gdk_cairo_set_source_color (cr, &style->bg[GTK_STATE_SELECTED]);
-      cairo_rectangle (cr, x, y, w, h);
-      cairo_fill (cr);
-#endif
-
-#if 0      
-      gtk_paint_focus (widget->style, widget->window,
-                       &event->area, widget, "meta-tab-image",
-                       x, y, w, h);
-#endif
-
-      cairo_destroy (cr);
     }
 
-  return GTK_WIDGET_CLASS (parent_class)->expose_event (widget, event);
+  return GTK_WIDGET_CLASS (parent_class)->draw (widget, cr);
 }
 
 #define META_TYPE_SELECT_WORKSPACE   (meta_select_workspace_get_type ())
@@ -819,8 +806,8 @@ unselect_workspace (GtkWidget *widget)
 
 static void meta_select_workspace_class_init (MetaSelectWorkspaceClass *klass);
 
-static gboolean meta_select_workspace_expose_event (GtkWidget      *widget,
-                                                    GdkEventExpose *event);
+static gboolean meta_select_workspace_draw (GtkWidget *widget,
+                                            cairo_t   *cr);
 
 GType
 meta_select_workspace_get_type (void)
@@ -858,7 +845,7 @@ meta_select_workspace_class_init (MetaSelectWorkspaceClass *klass)
   
   widget_class = GTK_WIDGET_CLASS (klass);
   
-  widget_class->expose_event = meta_select_workspace_expose_event;
+  widget_class->draw = meta_select_workspace_draw;
 }
 
 /**
@@ -896,15 +883,12 @@ meta_convert_meta_to_wnck (MetaWindow *window, MetaScreen *screen)
 
 
 static gboolean
-meta_select_workspace_expose_event (GtkWidget      *widget,
-                                    GdkEventExpose *event)
+meta_select_workspace_draw (GtkWidget *widget,
+                            cairo_t   *cr)
 {
   MetaWorkspace *workspace;
   WnckWindowDisplayInfo *windows;
   GtkAllocation allocation;
-  GtkStyle *style;
-  GdkWindow *window;
-  cairo_t *cr;
   int i, n_windows;
   GList *tmp, *list;
 
@@ -943,11 +927,10 @@ meta_select_workspace_expose_event (GtkWidget      *widget,
 
   g_list_free (list);
 
-  window = gtk_widget_get_window (widget);
   gtk_widget_get_allocation (widget, &allocation);
 
   wnck_draw_workspace (widget,
-                       window,
+                       cr,
                        SELECT_OUTLINE_WIDTH,
                        SELECT_OUTLINE_WIDTH,
                        allocation.width - SELECT_OUTLINE_WIDTH * 2,
@@ -963,20 +946,24 @@ meta_select_workspace_expose_event (GtkWidget      *widget,
   
   if (META_SELECT_WORKSPACE (widget)->selected)
     {
-      style = gtk_widget_get_style (widget);
-      cr = gdk_cairo_create (window);
+      GtkStyleContext *context;
+      GdkRGBA color;
+
+      context = gtk_widget_get_style_context (widget);
+
+      gtk_style_context_set_state (context,
+                                   gtk_widget_get_state_flags (widget));
+
+      gtk_style_context_lookup_color (context, "color", &color);
 
-      gdk_cairo_set_source_color (cr,
-                                  &style->fg[gtk_widget_get_state (widget)]);
       cairo_set_line_width (cr, SELECT_OUTLINE_WIDTH);
+      cairo_set_source_rgb (cr, color.red, color.green, color.blue);
 
       cairo_rectangle (cr,
                        SELECT_OUTLINE_WIDTH / 2.0, SELECT_OUTLINE_WIDTH / 2.0,
                        allocation.width - SELECT_OUTLINE_WIDTH,
                        allocation.height - SELECT_OUTLINE_WIDTH);
       cairo_stroke (cr);
-
-      cairo_destroy (cr);
     }
 
   return TRUE;
