@@ -17,9 +17,7 @@
  * General Public License for more details.
  * 
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
 #include "prefs.h"
@@ -34,8 +32,8 @@
 
 #include <string.h>
 #include <stdlib.h>
+#include <cairo-xlib.h>
 
-static void meta_stock_icons_init (void);
 static void meta_ui_accelerator_parse (const char      *accel,
                                        guint           *keysym,
                                        guint           *keycode,
@@ -58,10 +56,18 @@ struct _MetaUI
 void
 meta_ui_init (int *argc, char ***argv)
 {
+  /* As of 2.91.7, Gdk uses XI2 by default, which conflicts with the
+   * direct X calls we use - in particular, events caused by calls to
+   * XGrabPointer/XGrabKeyboard are no longer understood by GDK, while
+   * GDK will no longer generate the core XEvents we process.
+   * So at least for now, enforce the previous behavior.
+   */
+#if GTK_CHECK_VERSION(2, 91, 7)
+  gdk_disable_multidevice ();
+#endif
+
   if (!gtk_init_check (argc, argv))
     meta_fatal ("Unable to open X display %s\n", XDisplayName (NULL));
-
-  meta_stock_icons_init ();
 }
 
 Display*
@@ -89,8 +95,10 @@ static gboolean
 maybe_redirect_mouse_event (XEvent *xevent)
 {
   GdkDisplay *gdisplay;
+  GdkDeviceManager *gmanager;
+  GdkDevice *gdevice;
   MetaUI *ui;
-  GdkEvent gevent;
+  GdkEvent *gevent;
   GdkWindow *gdk_window;
   Window window;
 
@@ -116,19 +124,20 @@ maybe_redirect_mouse_event (XEvent *xevent)
   if (!ui)
     return FALSE;
 
-  gdk_window = gdk_window_lookup_for_display (gdisplay, window);
+  gdk_window = gdk_x11_window_lookup_for_display (gdisplay, window);
   if (gdk_window == NULL)
     return FALSE;
 
+  gmanager = gdk_display_get_device_manager (gdisplay);
+  gdevice = gdk_device_manager_get_client_pointer (gmanager);
+
   /* If GDK already thinks it has a grab, we better let it see events; this
    * is the menu-navigation case and events need to get sent to the appropriate
    * (client-side) subwindow for individual menu items.
    */
-  if (gdk_display_pointer_is_grabbed (gdisplay))
+  if (gdk_display_device_is_grabbed (gdisplay, gdevice))
     return FALSE;
 
-  memset (&gevent, 0, sizeof (gevent));
-
   switch (xevent->type)
     {
     case ButtonPress:
@@ -150,13 +159,13 @@ maybe_redirect_mouse_event (XEvent *xevent)
               ABS (xevent->xbutton.x - ui->button_click_x) <= double_click_distance &&
               ABS (xevent->xbutton.y - ui->button_click_y) <= double_click_distance)
             {
-              gevent.button.type = GDK_2BUTTON_PRESS;
+              gevent = gdk_event_new (GDK_2BUTTON_PRESS);
 
               ui->button_click_number = 0;
             }
           else
             {
-              gevent.button.type = GDK_BUTTON_PRESS;
+              gevent = gdk_event_new (GDK_BUTTON_PRESS);
               ui->button_click_number = xevent->xbutton.button;
               ui->button_click_window = xevent->xbutton.window;
               ui->button_click_time = xevent->xbutton.time;
@@ -166,28 +175,29 @@ maybe_redirect_mouse_event (XEvent *xevent)
         }
       else
         {
-          gevent.button.type = GDK_BUTTON_RELEASE;
+          gevent = gdk_event_new (GDK_BUTTON_RELEASE);
         }
 
-      gevent.button.window = gdk_window;
-      gevent.button.button = xevent->xbutton.button;
-      gevent.button.time = xevent->xbutton.time;
-      gevent.button.x = xevent->xbutton.x;
-      gevent.button.y = xevent->xbutton.y;
-      gevent.button.x_root = xevent->xbutton.x_root;
-      gevent.button.y_root = xevent->xbutton.y_root;
+      gevent->button.window = g_object_ref (gdk_window);
+      gevent->button.button = xevent->xbutton.button;
+      gevent->button.time = xevent->xbutton.time;
+      gevent->button.x = xevent->xbutton.x;
+      gevent->button.y = xevent->xbutton.y;
+      gevent->button.x_root = xevent->xbutton.x_root;
+      gevent->button.y_root = xevent->xbutton.y_root;
 
       break;
     case MotionNotify:
-      gevent.motion.type = GDK_MOTION_NOTIFY;
-      gevent.motion.window = gdk_window;
+      gevent = gdk_event_new (GDK_MOTION_NOTIFY);
+      gevent->motion.type = GDK_MOTION_NOTIFY;
+      gevent->motion.window = g_object_ref (gdk_window);
       break;
     case EnterNotify:
     case LeaveNotify:
-      gevent.crossing.type = xevent->type == EnterNotify ? GDK_ENTER_NOTIFY : GDK_LEAVE_NOTIFY;
-      gevent.crossing.window = gdk_window;
-      gevent.crossing.x = xevent->xcrossing.x;
-      gevent.crossing.y = xevent->xcrossing.y;
+      gevent = gdk_event_new (xevent->type == EnterNotify ? GDK_ENTER_NOTIFY : GDK_LEAVE_NOTIFY);
+      gevent->crossing.window = g_object_ref (gdk_window);
+      gevent->crossing.x = xevent->xcrossing.x;
+      gevent->crossing.y = xevent->xcrossing.y;
       break;
     default:
       g_assert_not_reached ();
@@ -195,7 +205,9 @@ maybe_redirect_mouse_event (XEvent *xevent)
     }
 
   /* If we've gotten here, we've filled in the gdk_event and should send it on */
-  gtk_main_do_event (&gevent);
+  gdk_event_set_device (gevent, gdevice);
+  gtk_main_do_event (gevent);
+  gdk_event_free (gevent);
 
   return TRUE;
 }
@@ -268,7 +280,10 @@ meta_ui_new (Display *xdisplay,
 
   g_assert (xdisplay == GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
   ui->frames = meta_frames_new (XScreenNumberOfScreen (screen));
-  gtk_widget_realize (GTK_WIDGET (ui->frames));
+  /* This does not actually show any widget. MetaFrames has been hacked so
+   * that showing it doesn't actually do anything. But we need the flags
+   * set for GTK to deliver events properly. */
+  gtk_widget_show (GTK_WIDGET (ui->frames));
 
   g_object_set_data (G_OBJECT (gdisplay), "meta-ui", ui);
 
@@ -315,7 +330,6 @@ meta_ui_create_frame_window (MetaUI *ui,
   gint attributes_mask;
   GdkWindow *window;
   GdkVisual *visual;
-  GdkColormap *cmap = gdk_screen_get_default_colormap (screen);
   
   /* Default depth/visual handles clients with weird visuals; they can
    * always be children of the root depth/visual obviously, but
@@ -328,7 +342,6 @@ meta_ui_create_frame_window (MetaUI *ui,
     {
       visual = gdk_x11_screen_lookup_visual (screen,
                                              XVisualIDFromVisual (xvisual));
-      cmap = gdk_colormap_new (visual, FALSE);
     }
 
   attrs.title = NULL;
@@ -344,7 +357,6 @@ meta_ui_create_frame_window (MetaUI *ui,
   attrs.y = y;
   attrs.wclass = GDK_INPUT_OUTPUT;
   attrs.visual = visual;
-  attrs.colormap = cmap;
   attrs.window_type = GDK_WINDOW_CHILD;
   attrs.cursor = NULL;
   attrs.wmclass_name = NULL;
@@ -354,7 +366,7 @@ meta_ui_create_frame_window (MetaUI *ui,
   attrs.width  = width;
   attrs.height = height;
 
-  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL | GDK_WA_COLORMAP;
+  attributes_mask = GDK_WA_X | GDK_WA_Y | GDK_WA_VISUAL;
 
   window =
     gdk_window_new (gdk_screen_get_root_window(screen),
@@ -390,8 +402,10 @@ meta_ui_map_frame   (MetaUI *ui,
                      Window  xwindow)
 {
   GdkWindow *window;
+  GdkDisplay *display;
 
-  window = gdk_xid_table_lookup (xwindow);
+  display = gdk_x11_lookup_xdisplay (ui->xdisplay);
+  window = gdk_x11_window_lookup_for_display (display, xwindow);
 
   if (window)
     gdk_window_show_unraised (window);
@@ -402,8 +416,10 @@ meta_ui_unmap_frame (MetaUI *ui,
                      Window  xwindow)
 {
   GdkWindow *window;
+  GdkDisplay *display;
 
-  window = gdk_xid_table_lookup (xwindow);
+  display = gdk_x11_lookup_xdisplay (ui->xdisplay);
+  window = gdk_x11_window_lookup_for_display (display, xwindow);
 
   if (window)
     gdk_window_hide (window);
@@ -495,219 +511,52 @@ meta_ui_window_menu_free (MetaWindowMenu *menu)
   meta_window_menu_free (menu);
 }
 
-struct _MetaImageWindow
-{
-  GtkWidget *window;
-  GdkPixmap *pixmap;
-};
-
-MetaImageWindow*
-meta_image_window_new (Display *xdisplay,
-                       int      screen_number,
-                       int      max_width,
-                       int      max_height)
-{
-  MetaImageWindow *iw;
-  GdkDisplay *gdisplay;
-  GdkScreen *gscreen;
-    
-  iw = g_new (MetaImageWindow, 1);
-  iw->window = gtk_window_new (GTK_WINDOW_POPUP);
-    
-  gdisplay = gdk_x11_lookup_xdisplay (xdisplay);
-  gscreen = gdk_display_get_screen (gdisplay, screen_number);
-  
-  gtk_window_set_screen (GTK_WINDOW (iw->window), gscreen);
- 
-  gtk_widget_realize (iw->window);
-  iw->pixmap = gdk_pixmap_new (gtk_widget_get_window (iw->window),
-                               max_width, max_height,
-                               -1);
-  
-  gtk_widget_set_size_request (iw->window, 1, 1);
-  gtk_widget_set_double_buffered (iw->window, FALSE);
-  gtk_widget_set_app_paintable (iw->window, TRUE);
-  
-  return iw;
-}
-
-void
-meta_image_window_free (MetaImageWindow *iw)
-{
-  gtk_widget_destroy (iw->window);
-  g_object_unref (G_OBJECT (iw->pixmap));
-  g_free (iw);
-}
-
-void
-meta_image_window_set_showing  (MetaImageWindow *iw,
-                                gboolean         showing)
-{
-  if (showing)
-    gtk_widget_show_all (iw->window);
-  else
-    {
-      gtk_widget_hide (iw->window);
-      meta_core_increment_event_serial (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()));
-    }
-}
-
-void
-meta_image_window_set (MetaImageWindow *iw,
-                       GdkPixbuf       *pixbuf,
-                       int              x,
-                       int              y)
-{
-  GdkWindow *window;
-  cairo_t *cr;
-
-  /* We use a back pixmap to avoid having to handle exposes, because
-   * it's really too slow for large clients being minimized, etc.
-   * and this way flicker is genuinely zero.
-   */
-
-  gdk_draw_pixbuf (iw->pixmap,
-                   gtk_widget_get_style (iw->window)->black_gc,
-		   pixbuf,
-                   0, 0,
-                   0, 0,
-                   gdk_pixbuf_get_width (pixbuf),
-                   gdk_pixbuf_get_height (pixbuf),
-                   GDK_RGB_DITHER_NORMAL,
-                   0, 0);
-  cr = gdk_cairo_create (iw->pixmap);
-  gdk_cairo_set_source_pixbuf (cr, pixbuf, 0, 0);
-  cairo_paint (cr);
-  cairo_destroy (cr);
-
-  window = gtk_widget_get_window (iw->window);
-  gdk_window_set_back_pixmap (window,
-                              iw->pixmap,
-                              FALSE);
-  
-  gdk_window_move_resize (window,
-                          x, y,
-                          gdk_pixbuf_get_width (pixbuf),
-                          gdk_pixbuf_get_height (pixbuf));
-
-  gdk_window_clear (window);
-}
-
-static GdkColormap*
-get_cmap (GdkPixmap *pixmap)
-{
-  GdkColormap *cmap;
-
-  cmap = gdk_drawable_get_colormap (pixmap);
-  if (cmap)
-    g_object_ref (G_OBJECT (cmap));
-
-  if (cmap == NULL)
-    {
-      if (gdk_drawable_get_depth (pixmap) == 1)
-        {
-          meta_verbose ("Using NULL colormap for snapshotting bitmap\n");
-          cmap = NULL;
-        }
-      else
-        {
-          meta_verbose ("Using system cmap to snapshot pixmap\n");
-          cmap = gdk_screen_get_system_colormap (gdk_drawable_get_screen (pixmap));
-
-          g_object_ref (G_OBJECT (cmap));
-        }
-    }
-
-  /* Be sure we aren't going to blow up due to visual mismatch */
-  if (cmap &&
-      (gdk_visual_get_depth (gdk_colormap_get_visual (cmap)) !=
-       gdk_drawable_get_depth (pixmap)))
-    {
-      cmap = NULL;
-      meta_verbose ("Switching back to NULL cmap because of depth mismatch\n");
-    }
-  
-  return cmap;
-}
-
 GdkPixbuf*
-meta_gdk_pixbuf_get_from_window (GdkPixbuf   *dest,
-                                 Window       xwindow,
+meta_gdk_pixbuf_get_from_pixmap (Pixmap       xpixmap,
                                  int          src_x,
                                  int          src_y,
-                                 int          dest_x,
-                                 int          dest_y,
                                  int          width,
                                  int          height)
 {
-  GdkDrawable *drawable;
+  cairo_surface_t *surface;
+  Display *display;
+  Window root_return;
+  int x_ret, y_ret;
+  unsigned int w_ret, h_ret, bw_ret, depth_ret;
+  XWindowAttributes attrs;
   GdkPixbuf *retval;
-  GdkColormap *cmap;
-  
-  retval = NULL;
-  
-  drawable = gdk_xid_table_lookup (xwindow);
 
-  if (drawable)
-    g_object_ref (G_OBJECT (drawable));
-  else
-    drawable = gdk_window_foreign_new (xwindow);
+  display = GDK_DISPLAY_XDISPLAY (gdk_display_get_default ());
 
-  cmap = get_cmap (drawable);
-  
-  retval = gdk_pixbuf_get_from_drawable (dest,
-                                         drawable,
-                                         cmap,
-                                         src_x, src_y,
-                                         dest_x, dest_y,
-                                         width, height);
+  if (!XGetGeometry (display, xpixmap, &root_return,
+                     &x_ret, &y_ret, &w_ret, &h_ret, &bw_ret, &depth_ret))
+    return NULL;
 
-  if (cmap)
-    g_object_unref (G_OBJECT (cmap));
-  g_object_unref (G_OBJECT (drawable));
-
-  return retval;
-}
-
-GdkPixbuf*
-meta_gdk_pixbuf_get_from_pixmap (GdkPixbuf   *dest,
-                                 Pixmap       xpixmap,
-                                 int          src_x,
-                                 int          src_y,
-                                 int          dest_x,
-                                 int          dest_y,
-                                 int          width,
-                                 int          height)
-{
-  GdkDrawable *drawable;
-  GdkPixbuf *retval;
-  GdkColormap *cmap;
-  
-  retval = NULL;
-  cmap = NULL;
-  
-  drawable = gdk_xid_table_lookup (xpixmap);
-
-  if (drawable)
-    g_object_ref (G_OBJECT (drawable));
-  else
-    drawable = gdk_pixmap_foreign_new (xpixmap);
-
-  if (drawable)
+  if (depth_ret == 1)
     {
-      cmap = get_cmap (drawable);
-  
-      retval = gdk_pixbuf_get_from_drawable (dest,
-                                             drawable,
-                                             cmap,
-                                             src_x, src_y,
-                                             dest_x, dest_y,
-                                             width, height);
+      surface = cairo_xlib_surface_create_for_bitmap (display,
+                                                      xpixmap,
+                                                      GDK_SCREEN_XSCREEN (gdk_screen_get_default ()),
+                                                      w_ret,
+                                                      h_ret);
     }
-  if (cmap)
-    g_object_unref (G_OBJECT (cmap));
-  if (drawable)
-    g_object_unref (G_OBJECT (drawable));
+  else
+    {
+      if (!XGetWindowAttributes (display, root_return, &attrs))
+        return NULL;
+
+      surface = cairo_xlib_surface_create (display,
+                                           xpixmap,
+                                           attrs.visual,
+                                           w_ret, h_ret);
+    }
+
+  retval = gdk_pixbuf_get_from_surface (surface,
+                                        src_x,
+                                        src_y,
+                                        width,
+                                        height);
+  cairo_surface_destroy (surface);
 
   return retval;
 }
@@ -795,8 +644,10 @@ meta_ui_window_should_not_cause_focus (Display *xdisplay,
                                        Window   xwindow)
 {
   GdkWindow *window;
+  GdkDisplay *display;
 
-  window = gdk_xid_table_lookup (xwindow);
+  display = gdk_x11_lookup_xdisplay (xdisplay);
+  window = gdk_x11_window_lookup_for_display (display, xwindow);
 
   /* we shouldn't cause focus if we're an override redirect
    * toplevel which is not foreign
@@ -811,17 +662,20 @@ char*
 meta_text_property_to_utf8 (Display             *xdisplay,
                             const XTextProperty *prop)
 {
+  GdkDisplay *display;
   char **list;
   int count;
   char *retval;
   
   list = NULL;
 
-  count = gdk_text_property_to_utf8_list (gdk_x11_xatom_to_atom (prop->encoding),
-                                          prop->format,
-                                          prop->value,
-                                          prop->nitems,
-                                          &list);
+  display = gdk_x11_lookup_xdisplay (xdisplay);
+  count = gdk_text_property_to_utf8_list_for_display (display,
+                                                      gdk_x11_xatom_to_atom_for_display (display, prop->encoding),
+                                                      prop->format,
+                                                      prop->value,
+                                                      prop->nitems,
+                                                      &list);
 
   if (count == 0)
     retval = NULL;
@@ -848,7 +702,8 @@ meta_ui_theme_get_frame_borders (MetaUI *ui,
   int text_height;
   PangoContext *context;
   const PangoFontDescription *font_desc;
-  GtkStyle *default_style;
+  GtkStyleContext *style = NULL;
+  PangoFontDescription *free_font_desc = NULL;
 
   if (meta_ui_have_a_theme ())
     {
@@ -857,8 +712,19 @@ meta_ui_theme_get_frame_borders (MetaUI *ui,
 
       if (!font_desc)
         {
-          default_style = gtk_widget_get_default_style ();
-          font_desc = default_style->font_desc;
+          GdkDisplay *display = gdk_x11_lookup_xdisplay (ui->xdisplay);
+          GdkScreen *screen = gdk_display_get_screen (display, XScreenNumberOfScreen (ui->xscreen));
+          GtkWidgetPath *widget_path;
+
+          style = gtk_style_context_new ();
+          gtk_style_context_set_screen (style, screen);
+          widget_path = gtk_widget_path_new ();
+          gtk_widget_path_append_type (widget_path, GTK_TYPE_WINDOW);
+          gtk_style_context_set_path (style, widget_path);
+          gtk_widget_path_free (widget_path);
+
+          gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL, "font", &free_font_desc, NULL);
+          font_desc = (const PangoFontDescription *) free_font_desc;
         }
 
       text_height = meta_pango_font_desc_get_text_height (font_desc, context);
@@ -867,11 +733,17 @@ meta_ui_theme_get_frame_borders (MetaUI *ui,
                                     type, text_height, flags,
                                     top_height, bottom_height,
                                     left_width, right_width);
+
+      if (free_font_desc)
+        pango_font_description_free (free_font_desc);
     }
   else
     {
       *top_height = *bottom_height = *left_width = *right_width = 0;
     }
+
+  if (style != NULL)
+    g_object_unref (style);
 }
 
 void
@@ -1082,9 +954,11 @@ gboolean
 meta_ui_window_is_widget (MetaUI *ui,
                           Window  xwindow)
 {
+  GdkDisplay *display;
   GdkWindow *window;
 
-  window = gdk_xid_table_lookup (xwindow);
+  display = gdk_x11_lookup_xdisplay (ui->xdisplay);
+  window = gdk_x11_window_lookup_for_display (display, xwindow);
 
   if (window)
     {
@@ -1096,48 +970,6 @@ meta_ui_window_is_widget (MetaUI *ui,
     return FALSE;
 }
 
-/* stock icon code Copyright (C) 2002 Jorn Baayen <jorn@nl.linux.org> */
-typedef struct
-{
-  char *stock_id;
-  const guint8 *icon_data;
-} MetaStockIcon;
-
-static void
-meta_stock_icons_init (void)
-{
-  GtkIconFactory *factory;
-  int i;
-
-  MetaStockIcon items[] =
-  {
-    { METACITY_STOCK_DELETE,   stock_delete_data   },
-    { METACITY_STOCK_MINIMIZE, stock_minimize_data },
-    { METACITY_STOCK_MAXIMIZE, stock_maximize_data }
-  };
-
-  factory = gtk_icon_factory_new ();
-  gtk_icon_factory_add_default (factory);
-
-  for (i = 0; i < (gint) G_N_ELEMENTS (items); i++)
-    {
-      GtkIconSet *icon_set;
-      GdkPixbuf *pixbuf;
-
-      pixbuf = gdk_pixbuf_new_from_inline (-1, items[i].icon_data,
-					   FALSE,
-					   NULL);
-
-      icon_set = gtk_icon_set_new_from_pixbuf (pixbuf);
-      gtk_icon_factory_add (factory, items[i].stock_id, icon_set);
-      gtk_icon_set_unref (icon_set);
-      
-      g_object_unref (G_OBJECT (pixbuf));
-    }
-
-  g_object_unref (G_OBJECT (factory));
-}
-
 int
 meta_ui_get_drag_threshold (MetaUI *ui)
 {
@@ -1160,31 +992,3 @@ meta_ui_get_direction (void)
 
   return META_UI_DIRECTION_LTR;
 }
-
-GdkPixbuf *
-meta_ui_get_pixbuf_from_pixmap (Pixmap   pmap)
-{
-  GdkPixmap *gpmap;
-  GdkScreen *screen;
-  GdkPixbuf *pixbuf;
-  GdkColormap *cmap;
-  int width, height, depth;
-
-  gpmap = gdk_pixmap_foreign_new (pmap);
-  screen = gdk_drawable_get_screen (gpmap);
-
-  gdk_drawable_get_size (GDK_DRAWABLE (gpmap), &width, &height);
-  
-  depth = gdk_drawable_get_depth (GDK_DRAWABLE (gpmap));
-  if (depth <= 24)
-    cmap = gdk_screen_get_system_colormap (screen);
-  else
-    cmap = gdk_screen_get_rgba_colormap (screen);
-  
-  pixbuf = gdk_pixbuf_get_from_drawable (NULL, gpmap, cmap, 0, 0, 0, 0,
-                                         width, height);
-
-  g_object_unref (gpmap);
-
-  return pixbuf;
-}
